import gzip
import os
import cPickle
import sys
import mx.DateTime
import LotusNotes
import copy
import util

count = 0
cacheCount = 0
cacheOutDir = None
pendingDocs = []

def main():

  global startTime
  global cacheOutDir
  global viewNoteIds

  if '-cache' in sys.argv:
    i = sys.argv.index('-cache')
    cacheOutDir = sys.argv[i+1]
    del sys.argv[i]
    del sys.argv[i]
  else:
    cacheOutDir = None

  if '-items' in sys.argv:
    i = sys.argv.index('-items')
    itemsList = sys.argv[i+1].split()
    del sys.argv[i]
    del sys.argv[i]
    print 'Will export just these items for each Note: %s' % (', '.join(itemsList))
  else:
    itemsList = None
    print 'Will export all items for each Note!'

  if '-view' in sys.argv:
    i = sys.argv.index('-view')
    viewName = sys.argv[i+1]
    del sys.argv[i]
    del sys.argv[i]
    print 'Will export notes in the view "%s"' % viewName
  else:
    print 'No view name'
    viewName = None

  if '-formula' in sys.argv:
    i = sys.argv.index('-formula')
    formula = sys.argv[i+1]
    del sys.argv[i]
    del sys.argv[i]
    print 'Will export notes with the formula "%s"' % formula
  else:
    print 'No formula'
    formula = None

  if '-docLimit' in sys.argv:
    i = sys.argv.index('-docLimit')
    docLimit = int(sys.argv[i+1])
    del sys.argv[i]
    del sys.argv[i]
    print 'Will export first %d notes' % docLimit
  else:
    print 'No docLimit'
    docLimit = -1

  if len(sys.argv) not in (2, 3):
    print
    print 'Usage: %s dbPath [serverName] [-cache outDirName] [-items itemsList] [-view viewName] [-formula formula] [-docLimit N]' % sys.argv[0]
    print
    sys.exit(1)

  dbPath = sys.argv[1]
  if len(sys.argv) == 3:
    serverName = sys.argv[2]
  else:
    # Local file export:
    serverName = ''

  startTime = mx.DateTime.now()
  #LotusNotes.export('', 'c:/tmp/Abbott.nsf', callback, ['CommentLine', 'pmrid', 'keyword1', 'body', 'country', 'branch'], None)
  #LotusNotes.export('MikeTest/iPhrase', 'Abbott2', callback, ['CommentLine', 'pmrid', 'keyword1', 'body', 'country', 'branch'], None)
  #LotusNotes.export('', 'c:/tmp/LotusKB.nsf', callback, ['abstract', 'Content', 'DocNumber', 'Subject', 'AdditionalKeywords'], None)
  #LotusNotes.export('', 'c:/tmp/LotusKB.nsf', callback, None, None)
  #LotusNotes.export('c:/tmp/notesapi/notesapi/doc/api65ug.nsf', callback, ['SectionName', 'SectionNumber', 'text'])
  #LotusNotes.export('MikeTest/iPhrase', 'retain_full2.nsf', callback, ['CommentLine', 'pmrid', 'keyword1', 'body', 'country', 'branch'], None)

  if cacheOutDir is not None:
    theCallback = cacheCallback
    # C api expects this:
    if not os.path.exists(cacheOutDir):
      os.makedirs(cacheOutDir, 0777)
    attachmentsDir = os.path.join(cacheOutDir, 'attachments/').replace('\\', '/')
    if not os.path.exists(attachmentsDir):
      os.makedirs(attachmentsDir, 0777)
  else:
    theCallback = countCallback
    attachmentsDir = None

  # Not the most efficient approach because we pull all view IDs
  # upfront rather than making it part of the NSFSearch -- but can't
  # figure out how to do that just yet:
  if viewName != None:
    print
    print 'Getting NoteIDs for view "%s"...' % viewName
    viewNoteIds = {}
    for noteId in LotusNotes.getViewNoteIds(serverName, dbPath, viewName):
      viewNoteIds[noteId] = 1
    print '  %d notes in view' % len(viewNoteIds)
    print 'Done'
    if itemsList != None and '__NoteId' not in itemsList:
      itemsList.append('__NoteId')
  else:
    viewNoteIds = None

  print
  print 'Now export...'
  # viewName = None
  LotusNotes.export(serverName, dbPath, attachmentsDir, theCallback, itemsList, None, formula, viewName, docLimit)

  if cacheOutDir and len(pendingDocs) > 0:
    flushDocs()

  print '%s: TOTAL %d' % (str(mx.DateTime.now()-startTime), count)

def cacheCallback(doc):
  
  """
  Called once per doc seen by the connector.
  """

  if viewNoteIds is None or doc['__NoteId'] in viewNoteIds:

    global count
    count += 1

    if 0 and '$FILE' in doc:
      print 'ATTACHMENTS: %s' % doc['$FILE']

    util.unPageAttachments(doc)

    pendingDocs.append(copy.copy(doc))

    if count % 1000 == 0:
      # print '%s: %d [%d pmrs]...' % (str(mx.DateTime.now()-startTime), count, len(pmrData))
      print '%s: %d' % (str(mx.DateTime.now()-startTime), count)
      flushDocs()

def flushDocs():
  global cacheCount
  global pendingDocs
  fileName = '%s/%d.pk.gz' % (cacheOutDir, cacheCount)
  f = gzip.GzipFile(fileName, 'wb')
  cPickle.dump(pendingDocs, f, 1)
  f.close()
  pendingDocs = []
  cacheCount += 1
  
allStatus = {}

def countCallback(doc):

  """
  Called once per doc seen by the connector.
  """

  if viewNoteIds is None or doc['__NoteId'] in viewNoteIds:
    global count
    count += 1

    if count % 1000 == 0:
      # print '%s: %d [%d pmrs]...' % (str(mx.DateTime.now()-startTime), count, len(pmrData))
      print '%s: %d' % (str(mx.DateTime.now()-startTime), count)
  
if __name__ == '__main__':
  main()
